// Copyright 2010-2020, Google Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef MOZC_BASE_STOPWATCH_H_
#define MOZC_BASE_STOPWATCH_H_

#include "base/port.h"

namespace mozc {

class Stopwatch {
 public:
  enum StopwatchState {
    STOPWATCH_RUNNING,
    STOPWATCH_STOPPED,
  };

  // Initializes a new stopwatch object, starts measuring elapsed time,
  // and returns the stopwatch object.
  static Stopwatch StartNew();

  Stopwatch();

  // Sets elapsed time to zero.
  // Stops the time counting.
  void Reset();

  // (Re)starts the time counting.
  // This function does not reset elapsed time so that you can use Start()
  // to accumulate a periodic activity.
  void Start();

  // Stops the time counting.
  // This function does not reset elapsed time so that you can use Start()
  // to accumulate a periodic activity.
  void Stop();

  // Returns the elapsed time as millisecond.
  int64 GetElapsedMilliseconds();

  // Returns the elapsed time as microsecond.
  double GetElapsedMicroseconds();

  // Returns the elapsed time as nanosecond.
  double GetElapsedNanoseconds();

  // Returns the elapsed time as ticks.
  int64 GetElapsedTicks();

  // Returns true if the time counting is on-going.
  bool IsRunning() const;

 private:
  StopwatchState state_;
  int64 frequency_;
  int64 start_timestamp_;
  int64 elapsed_timestamp_;
};

}  // namespace mozc

#endif  // MOZC_BASE_STOPWATCH_H_
